@@ -3,6 +3,7 @@ from preql.compiler import cast_to_instance
 import re
 import csv
 import inspect
+import itertools
 from typing import Optional
 
 from tqdm import tqdm
@@ -39,6 +40,7 @@ def _pql_PY_callback(state: State, var: str):
 
 
 def pql_PY(state: State, code_expr: T.string, code_setup: T.string.as_nullable() = objects.null):
+    "Run the given Python code and convert the result to a Preql object"
     py_code = cast_to_python(state, code_expr)
     py_setup = cast_to_python(state, code_setup)
 
@@ -67,6 +69,8 @@ def pql_inspect_sql(state: State, obj: T.object):
 
 
 def pql_SQL(state: State, result_type: T.union[T.collection, T.type], sql_code: T.string):
+    """Create an object with the given SQL evaluation code, and given result type
+    """
     # TODO optimize for when the string is known (prefetch the variables and return Sql)
     # .. why not just compile with parameters? the types are already known
     return ast.ParameterizedSqlCode(result_type, sql_code)
@@ -121,6 +125,7 @@ def pql_brk_continue(state):
 
 
 def pql_breakpoint(state: State):
+    "Create a Python breakpoint. For a Preql breakpoint, use debug() instead"
     breakpoint()
     return objects.null
 
@@ -188,8 +193,7 @@ def pql_temptable(state: State, expr: T.collection, const: T.bool.as_nullable()
 
 
 def pql_get_db_type(state: State):
-    """
-    Returns a string representing the type of the active database.
+    """Returns a string representing the type of the active database.
 
     Possible values are:
         - "sqlite"
@@ -244,61 +248,86 @@ def pql_table_concat(state: State, t1: T.collection, t2: T.collection):
     return sql_bin_op(state, "UNION ALL", t1, t2, "concatenate", True)
 
 
+def _get_table(t):
+    if isinstance(t, objects.SelectedColumnInstance):
+        return t.parent
 
+    assert isinstance(t, objects.CollectionInstance)
+    return t
 
-def _join(state: State, join: str, exprs: dict, joinall=False, nullable=None):
+def _join2(state, join, a, b):
+    if isinstance(a, objects.SelectedColumnInstance) and isinstance(b, objects.SelectedColumnInstance):
+        return [a, b]
 
-    exprs = {name: evaluate(state, value) for name, value in exprs.items()}
-    for x in exprs.values():
+    if not ((a.type <= T.collection) and (b.type <= T.collection)):
+        raise Signal.make(T.TypeError, state, None, f"join() arguments must be of same type. Instead got:\n * {a}\n * {b}")
+
+    return _auto_join(state, join, a, b)
+
+def _join(state: State, join: str, exprs_dict: dict, joinall=False, nullable=None):
+
+    names = list(exprs_dict)
+    exprs = [evaluate(state, value) for value in exprs_dict.values()]
+
+    # Validation and edge cases
+    for x in exprs:
         if not isinstance(x, objects.AbsInstance):
             raise Signal.make(T.TypeError, state, None, f"Unexpected object type: {x}")
 
-    if len(exprs) != 2:
-        raise Signal.make(T.TypeError, state, None, "join expected only 2 arguments")
+    for e in exprs:
+        if e is objects.EmptyList:
+            raise Signal.make(T.TypeError, state, None, "Cannot join on an untyped empty list")
 
-    (a,b) = exprs.values()
-
-    if a is objects.EmptyList or b is objects.EmptyList:
-        raise Signal.make(T.TypeError, state, None, "Cannot join on an untyped empty list")
-
-    if isinstance(a, objects.UnknownInstance) or isinstance(b, objects.UnknownInstance):
-        table_type = T.table({e: T.unknown for e in exprs})
-        return objects.TableInstance.make(sql.unknown, table_type, [])
-
-    if isinstance(a, objects.SelectedColumnInstance) and isinstance(b, objects.SelectedColumnInstance):
-        cols = a, b
-        tables = [a.parent, b.parent]
-    else:
-        if not ((a.type <= T.collection) and (b.type <= T.collection)):
-            raise Signal.make(T.TypeError, state, None, f"join() got unexpected values:\n * {a}\n * {b}")
-        if joinall:
-            tables = (a, b)
-        else:
-            cols = _auto_join(state, join, a, b)
-            tables = [c.parent for c in cols]
+        if isinstance(e, objects.UnknownInstance):
+            table_type = T.table({n: T.unknown for n in names})
+            return objects.TableInstance.make(sql.unknown, table_type, [])
 
+    # Initialization
+    tables = [_get_table(x) for x in exprs]
     assert all((t.type <= T.collection) for t in tables)
 
-    structs = {name: T.struct(table.type.elems) for name, table in safezip(exprs, tables)}
+    structs = {name: T.struct(table.type.elems) for name, table in safezip(names, tables)}
 
-    # Update nullable for left/right/outer joins
     if nullable:
+        # Update nullable for left/right/outer joins
         structs = {name: t.as_nullable() if n else t
                    for (name, t), n in safezip(structs.items(), nullable)}
 
-    tables = [objects.alias_table_columns(t, n) for n, t in safezip(exprs, tables)]
+    tables = [objects.alias_table_columns(t, n) for n, t in safezip(names, tables)]
 
     primary_keys = [ [name] + pk
-                    for name, t in safezip(exprs, tables)
+                    for name, t in safezip(names, tables)
                     for pk in t.type.options.get('pk', [])
                 ]
     table_type = T.table(structs, name=Id(state.unique_name("joinall" if joinall else "join")), pk=primary_keys)
 
-    conds = [] if joinall else [sql.Compare('=', [sql.Name(c.type, join_names((n, c.name))) for n, c in safezip(structs, cols)])]
+    conds = []
+    if joinall:
+        for e in exprs:
+            if not isinstance(e, objects.CollectionInstance):
+                raise Signal.make(T.TypeError, state, None, f"joinall() expected tables. Got {e}")
+    else:
+        if len(exprs) < 2:
+            raise Signal.make(T.TypeError, state, None, "join expected at least 2 arguments")
+
+        joined_exprs = set()
+        for (na, ta), (nb, tb) in itertools.combinations(safezip(names, exprs), 2):
+            try:
+                cols = _join2(state, join, ta, tb)
+                conds.append( sql.Compare('=', [sql.Name(c.type, join_names((n, c.name))) for n, c in safezip([na, nb], cols)]) )
+                joined_exprs |= {id(ta), id(tb)}
+            except NoAutoJoinFound as e:
+                pass
+
+        if {id(e) for e in exprs} != set(joined_exprs):
+            # TODO better error!!! table name?? specific failed auto-join?
+            s = ', '.join(repr(t.type) for t in exprs)
+            raise Signal.make(T.JoinError, state, None, f"Cannot auto-join: No plausible relations found between {s}")
+
 
     code = sql.Join(table_type, join, [t.code for t in tables], conds)
+    return objects.TableInstance.make(code, table_type, exprs)
 
-    return objects.TableInstance.make(code, table_type, [a,b])
 
 def pql_join(state, tables):
     "Inner join two tables into a new projection {t1, t2}"
@@ -313,12 +342,17 @@ def pql_joinall(state: State, tables):
     "Cartesian product of two tables into a new projection {t1, t2}"
     return _join(state, "JOIN", tables, True)
 
+class NoAutoJoinFound(Exception):
+    pass
+
 def _auto_join(state, join, ta, tb):
+    # if len(ta.type.elems) == len(tb.type.elems) == 1:
+
     refs1 = _find_table_reference(ta, tb)
     refs2 = _find_table_reference(tb, ta)
     auto_join_count = len(refs1) + len(refs2)
     if auto_join_count < 1:
-        raise Signal.make(T.JoinError, state, None, "Cannot auto-join: No plausible relations found")
+        raise NoAutoJoinFound(ta, tb)
     elif auto_join_count > 1:   # Ambiguity in auto join resolution
         raise Signal.make(T.JoinError, state, None, "Cannot auto-join: Several plausible relations found")
 
@@ -334,22 +368,19 @@ def _auto_join(state, join, ta, tb):
 
 @listgen
 def _find_table_reference(t1, t2):
-    # XXX TODO need to check TableType too (owner)?
     for name, c in t1.type.elems.items():
         if (c <= T.t_relation):
-            if c.elem == t2.type:
-                # TODO depends on the query XXX
-                yield (objects.SelectedColumnInstance(t2, T.t_id, 'id'), objects.SelectedColumnInstance(t1, c, name))
+            rel = c.options['rel']
+            if rel['table'] == t2.type:     # if same table
+                yield t2.get_attr(rel['column']), objects.SelectedColumnInstance(t1, c, name)
 
 def pql_type(state: State, obj: T.any):
-    """
-    Returns the type of the given object
+    """Returns the type of the given object
     """
     return obj.type
 
 def pql_repr(state: State, obj: T.any):
-    """
-    Returns the type of the given object
+    """Returns the repr of the given object
     """
     try:
         return new_value_instance(obj.repr(state))
@@ -358,8 +389,7 @@ def pql_repr(state: State, obj: T.any):
         return new_value_instance(value)
 
 def pql_columns(state: State, table: T.collection):
-    """
-    Returns a dictionary of {column_name: column_type}
+    """Accepts a table, and returns a dictionary of {column_name: column_type}
     """
     elems = table.type.elems
     if isinstance(elems, tuple):    # Create a tuple/list instead of dict?
@@ -406,17 +436,22 @@ def pql_import_table(state: State, name: T.string, columns: T.list[T.string] = o
 
 
 
-def pql_connect(state: State, uri: T.string):
-    """
-    Connect to a new database, specified by the uri
+def pql_connect(state: State, uri: T.string, load_all_tables: T.bool = ast.false, auto_create: T.bool = ast.false):
+    """Connect to a new database, specified by the uri
+
+    If load_all_tables is true, loads all the tables in the database to the global namespace.
+    If auto_create is true, create the database if it doesn't exist (Sqlite only)
     """
     uri = cast_to_python(state, uri)
-    state.connect(uri)
+    load_all_tables = cast_to_python(state, load_all_tables)
+    auto_create = cast_to_python(state, auto_create)
+    state.connect(uri, auto_create=auto_create)
+    if load_all_tables:
+        state._py_api.load_all_tables()     # XXX
     return objects.null
 
 def pql_help(state: State, inst: T.any = objects.null):
-    """
-    Provides a brief summary for a given object
+    """Provides a brief summary for a given object
     """
     if inst is objects.null:
         text = (
@@ -443,6 +478,12 @@ def pql_help(state: State, inst: T.any = objects.null):
     text = '\n'.join(lines) + '\n'
     return new_value_instance(text).replace(type=T.text)
 
+def _get_doc(v):
+    s = ''
+    if v.type <= T.function and v.docstring:
+        s = v.docstring.splitlines()[0]
+    return new_str(s).code
+
 def pql_names(state: State, obj: T.any = objects.null):
     """List all names in the namespace of the given object.
 
@@ -454,13 +495,16 @@ def pql_names(state: State, obj: T.any = objects.null):
         all_vars = obj.all_attrs()
 
     assert all(isinstance(s, str) for s in all_vars)
-    tuples = [sql.Tuple(T.list[T.string], [new_str(n).code,new_str(v.type).code]) for n,v in all_vars.items()]
+    all_vars = list(all_vars.items())
+    all_vars.sort()
+    tuples = [sql.Tuple(T.list[T.string], [new_str(n).code,new_str(v.type).code, _get_doc(v)]) for n,v in all_vars]
 
-    table_type = T.table(dict(name=T.string, type=T.string))
+    table_type = T.table(dict(name=T.string, type=T.string, doc=T.string))
     return objects.new_const_table(state, table_type, tuples)
 
 
 def pql_tables(state: State):
+    "Return a table of all tables in the database"
     names = state.db.list_tables()
     values = [(name, state.db.import_table_type(state, name, None)) for name in names]
     tuples = [sql.Tuple(T.list[T.string], [new_str(n).code,new_str(t).code]) for n,t in values]
@@ -567,6 +611,7 @@ def _rest_table_endpoint(state, table):
 
 
 def pql_serve_rest(state: State, endpoints: T.struct, port: T.int = new_int(8080)):
+    "Start a starlette server that exposes the current namespace as REST API"
 
     try:
         from starlette.applications import Starlette
