@@ -242,6 +242,7 @@ class Instance(AbsInstance):
 
 
 def new_value_instance(value, type_=None, force_type=False):
+
     r = sql.make_value(value)
 
     if force_type:
@@ -611,6 +612,8 @@ class PythonList(ast.Ast):
     # TODO just a regular const?
     def __init__(self, items):
         types = set(type(i) for i in items)
+        if len(types) > 1:
+            raise ValueError("Expecting all items of the list to be of the same type")
         # TODO if not one type, raise typeerror
         type_ ,= types
         self.type = T.list[pql_types.from_python(type_)]
